# locust error
not_found_locust_error = "locust not found"
# json error
cant_reformat_json_error = "can't reformat json is type right?"
wrong_json_data_error = "can't parser json"
cant_find_json_error = "can't find json"
cant_save_json_error = "can't save json"
# executor error
executor_data_error = "executor receive wrong data"
executor_list_error = "executor receive wrong data list is none or wrong type"
# HTML
html_generate_no_data_tag = "record is None"



class LocustNotFoundException(Exception):
    pass


class JELoadingTestException(Exception):
    pass


class JELoadingTestJsonException(JELoadingTestException):
    pass


class JELoadingTestExecuteException(JELoadingTestException):
    pass


class JELoadingAssertException(JELoadingTestException):
    pass


class HTMLException(JELoadingTestException):
    pass



# env
from je_load_density.wrapper.locust_as_library.locust_as_library import create_env

# user
from je_load_density.wrapper.locust_template.http_user_with_api_testka import create_loading_test_user
from je_load_density.wrapper.locust_template.http_user_with_api_testka import HttpUserWrapper

# start
from je_load_density.wrapper.locust_as_library.locust_as_library import start_test
from je_load_density.wrapper.env_with_user.wrapper_env_and_user import loading_test_with_user

# executor
from je_load_density.utils.executor.action_executor import execute_action
from je_load_density.utils.executor.action_executor import execute_files
# file
from je_load_density.utils.file_process.get_dir_file_list import get_dir_files_as_list
# report
from je_load_density.utils.html_report.html_report_generate import generate_html




class TestRecord(object):
    """
    data class to record success and failure test
    """

    def __init__(self):
        self.record_list = list()
        self.error_record_list = list()

    def clean_record(self):
        self.record_list = list()
        self.error_record_list = list()


test_record = TestRecord()




from je_load_density.wrapper.locust_template.http_user_with_api_testka import create_loading_test_user
from je_load_density.wrapper.locust_as_library.locust_as_library import start_test


def loading_test_with_user(
        user_detail_dict: dict,
        user_count: int = 50, spawn_rate: int = 10, test_time: int = 60,
        web_ui_dict: dict = None,
        **kwargs
):
    user = create_loading_test_user(user_detail_dict)
    start_test(
        user, user_count, spawn_rate, test_time,
        web_ui_dict, **kwargs
    )



# argparse
import argparse

from je_load_density.utils.json.json_file.json_file import read_action_json
from je_load_density.utils.file_process.get_dir_file_list import get_dir_files_as_list
from je_load_density.utils.executor.action_executor import execute_action
from je_load_density.utils.executor.action_executor import execute_files


def preprocess_execute_action(file_path: str):
    execute_action(read_action_json(file_path))


def preprocess_execute_files(file_path: str):
    execute_files(get_dir_files_as_list(file_path))


argparse_event_dict = {
    "execute_file": preprocess_execute_action,
    "execute_dir": preprocess_execute_files
}

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("-e", "--execute_file", type=str, help="choose action file to execute")
    parser.add_argument("-d", "--execute_dir", type=str, help="choose dir include action file to execute")
    args = parser.parse_args()
    args = vars(args)
    print(args)
    for key, value in args.items():
        if value is not None:
            argparse_event_dict.get(key)(value)



import gevent
from locust.env import Environment
from locust.stats import stats_printer, stats_history
from locust.log import setup_logging
from locust import User
from locust import events

from je_load_density.utils.test_record.record_test_result_class import test_record

setup_logging("INFO", None)


@events.request.add_listener
def handle_request(request_type, name, response_time, response_length, response,
                   context, exception, start_time, url, **kwargs):
    # TODO
    if exception:
        test_record.error_record_list.append(
                {
                    "http_method": request_type,
                    "test_url": url,
                    "name": name,
                    "status_code": response.status_code,
                    "error": exception
                 }
        )
    else:
        test_record.record_list.append(
                {
                    "http_method": request_type,
                    "test_url": url,
                    "name": name,
                    "status_code": response.status_code,
                    "text": response.text,
                    "content": response.content,
                    "headers": response.headers,
                }
        )


def create_env(user_class: [User]):
    env = Environment(user_classes=[user_class], events=events)
    env.create_local_runner()
    gevent.spawn(stats_printer(env.stats))
    gevent.spawn(stats_history, env.runner)
    return env


def start_test(user_class: [User], user_count: int = 50, spawn_rate: int = 10, test_time: int = 60,
               web_ui_dict: dict = None,
               **kwargs):
    env = create_env(user_class)
    env.runner.start(user_count, spawn_rate=spawn_rate)
    if web_ui_dict is not None:
        env.create_web_ui(web_ui_dict.get("host", "127.0.0.1"), web_ui_dict.get("port", "8089"))
    if test_time is not None:
        gevent.spawn_later(test_time, lambda: env.runner.quit())
    env.runner.greenlet.join()
    if web_ui_dict is not None:
        env.web_ui.stop()



import sys

from je_load_density.utils.exception.exception import JELoadingTestExecuteException
from je_load_density.wrapper.env_with_user.wrapper_env_and_user import loading_test_with_user
from je_load_density.utils.exception.exception_tag import executor_data_error
from je_load_density.utils.exception.exception_tag import executor_list_error
from je_load_density.utils.json.json_file.json_file import read_action_json


event_dict = {
    "loading_test_with_user": loading_test_with_user,
}


def execute_event(action: list):
    """
    :param action: execute action
    :return: what event return
    """
    event = event_dict.get(action[0])
    if len(action) == 2:
        return event(**action[1])
    else:
        raise JELoadingTestExecuteException(executor_data_error)


def execute_action(action_list: list):
    """
    :param action_list: like this structure
    [
        ["method on event_dict", {"param": params}],
        ["method on event_dict", {"param": params}]
    ]
    for loop and use execute_event function to execute
    :return: recode string, response as list
    """
    execute_record_string = ""
    event_response_list = []
    try:
        if len(action_list) > 0 or type(action_list) is not list:
            for action in action_list:
                event_response = execute_event(action)
                print("execute: ", str(action))
                execute_record_string = "".join(execute_record_string)
                event_response_list.append(event_response)
        else:
            raise JELoadingTestExecuteException(executor_list_error)
        return execute_record_string, event_response_list
    except Exception as error:
        print(repr(error), file=sys.stderr)


def execute_files(execute_files_list: list):
    """
    :param execute_files_list: list include execute files path
    :return: every execute detail as list
    """
    execute_detail_list = list()
    for file in execute_files_list:
        execute_detail_list.append(execute_action(read_action_json(file)))
    return execute_detail_list



import setuptools

with open("README.md", "r") as README:
    long_description = README.read()

setuptools.setup(
    name="je_load_density",
    version="0.0.03",
    author="JE-Chen",
    author_email="zenmailman@gmail.com",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/JE-Chen/LoadTesting",
    packages=setuptools.find_packages(),
    install_requires=[
        "locust"
    ],
    classifiers=[
        "Programming Language :: Python :: 3.5",
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ]
)

# python setup.py sdist bdist_wheel
# python -m twine upload dist/*



import json
import os.path
from pathlib import Path
from threading import Lock

from je_load_density.utils.exception.exception import JELoadingTestJsonException
from je_load_density.utils.exception.exception_tag import cant_find_json_error
from je_load_density.utils.exception.exception_tag import cant_save_json_error


lock = Lock()


def read_action_json(json_file_path: str):
    """
    :param json_file_path json file's path to read
    """
    try:
        lock.acquire()
        file_path = Path(json_file_path)
        if file_path.exists() and file_path.is_file():
            with open(json_file_path) as read_file:
                return json.load(read_file)
    except JELoadingTestJsonException:
        raise JELoadingTestJsonException(cant_find_json_error)
    finally:
        lock.release()


def write_action_json(json_save_path: str, action_json: list):
    """
    :param json_save_path  json save path
    :param action_json the json str include action to write
    """
    try:
        lock.acquire()
        with open(json_save_path, "w+") as file_to_write:
            file_to_write.write(json.dumps(action_json))
    except JELoadingTestJsonException:
        raise JELoadingTestJsonException(cant_save_json_error)
    finally:
        lock.release()




import sys

from je_load_density.utils.test_record.record_test_result_class import test_record
from je_load_density.utils.exception.exception import HTMLException
from je_load_density.utils.exception.exception_tag import html_generate_no_data_tag
from threading import Lock

lock = Lock()

html_string = \
    r"""
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8"/>
    <title>Load Density Report</title>

    <style>
      
        body{{
            font-size: 100%;
        }}

        h1{{
            font-size: 2em;
        }}

        .main_table {{
            margin: 0 auto;
            border-collapse: collapse;
            width: 75%;
            font-size: 1.5em;
        }}

        .success_table_head {{
            border: 3px solid #262626;
            background-color: aqua;
            font-family: "Times New Roman", sans-serif;
            text-align: center;
        }}

        .failure_table_head {{
            border: 3px solid #262626;
            background-color: #f84c5f;
            font-family: "Times New Roman", sans-serif;
            text-align: center;
        }}

        .table_data_field_title {{
            border: 3px solid #262626;
            padding: 0;
            margin: 0;
            background-color: #dedede;
            font-family: "Times New Roman", sans-serif;
            text-align: center;
            width: 25%;
        }}

        .table_data_field_text {{
            border: 3px solid #262626;
            padding: 0;
            margin: 0;
            background-color: #dedede;
            font-family: "Times New Roman", sans-serif;
            text-align: left;
            width: 75%;
        }}

        .text {{
            text-align: center;
            font-family: "Times New Roman", sans-serif;
        }}
    </style>
</head>
<body>
<h1 class="text">
    Test Report
</h1>
{success_table}
{failure_table}
</body>
</html>
""".strip()

success_table = \
    r"""
   <table class="main_table">
    <thead>
    <tr>
        <th colspan="2" class="success_table_head">Test Report</th>
    </tr>
    </thead>
    <tbody>
    <tr>
        <td class="table_data_field_title">Method</td>
        <td class="table_data_field_text">{Method}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">test_url</td>
        <td class="table_data_field_text">{test_url}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">name</td>
        <td class="table_data_field_text">{name}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">status_code</td>
        <td class="table_data_field_text">{status_code}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">text</td>
        <td class="table_data_field_text">{text}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">content</td>
        <td class="table_data_field_text">{content}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">headers</td>
        <td class="table_data_field_text">{headers}</td>
    </tr>
    </tbody>
</table>
    <br>
    """.strip()

failure_table = \
    r"""
<table class="main_table">
    <thead>
    <tr>
        <th colspan="2" class="failure_table_head">Test Report</th>
    </tr>
    </thead>
    <tbody>
    <tr>
        <td class="table_data_field_title">http_method</td>
        <td class="table_data_field_text">{http_method}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">test_url</td>
        <td class="table_data_field_text">{test_url}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">name</td>
        <td class="table_data_field_text">{name}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">status_code</td>
        <td class="table_data_field_text">{status_code}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">error</td>
        <td class="table_data_field_text">{error}</td>
    </tr>
    </tbody>
</table>
<br>
    """.strip()


def generate_html(html_name: str = "default_name"):
    """
    :param html_name: save html file name
    :return: html_string
    """
    if len(test_record.record_list) == 0 and len(test_record.error_record_list) == 0:
        raise HTMLException(html_generate_no_data_tag)
    else:
        success = ""
        for record_data in test_record.record_list:
            success = "".join(
                [
                    success,
                    success_table.format(
                        Method=record_data.get("http_method"),
                        test_url=record_data.get("test_url"),
                        name=record_data.get("name"),
                        status_code=record_data.get("status_code"),
                        text=record_data.get("text"),
                        content=record_data.get("content"),
                        headers=record_data.get("headers"),
                    )
                ]
            )
        failure = ""
        if len(test_record.error_record_list) == 0:
            pass
        else:
            for record_data in test_record.error_record_list:
                failure = "".join(
                    [
                        failure,
                        failure_table.format(
                            http_method=record_data[0].get("http_method"),
                            test_url=record_data[0].get("test_url"),
                            name=record_data[0].get("name"),
                            status_code=record_data[0].get("status_code"),
                            error=record_data[0].get("error"),
                        ),
                    ]
                )
        new_html_string = html_string.format(success_table=success, failure_table=failure)
        try:
            lock.acquire()
            with open(html_name + ".html", "w+") as file_to_write:
                file_to_write.write(
                    new_html_string
                )
        except Exception as error:
            print(repr(error), file=sys.stderr)
        finally:
            lock.release()
    return new_html_string
